@@ -5,8 +5,9 @@ from dataclasses import dataclass
 from functools import cache, cached_property, lru_cache, partial, wraps
 from itertools import chain
 from operator import neg
+from pathlib import Path
 from types import NoneType
-from typing import TYPE_CHECKING, Any, ClassVar, ParamSpec, TypeVar, cast
+from typing import TYPE_CHECKING, Any, ClassVar, cast
 
 from hypothesis import given
 from hypothesis.strategies import (
@@ -23,25 +24,26 @@ from hypothesis.strategies import (
 )
 from pytest import raises
 
-from utilities.datetime import ZERO_TIME, get_now, get_today
 from utilities.errors import ImpossibleCaseError
 from utilities.functions import (
     EnsureBoolError,
     EnsureBytesError,
     EnsureClassError,
     EnsureDateError,
-    EnsureDateTimeError,
     EnsureFloatError,
     EnsureHashableError,
     EnsureIntError,
     EnsureMemberError,
     EnsureNotNoneError,
     EnsureNumberError,
+    EnsurePathError,
+    EnsurePlainDateTimeError,
     EnsureSizedError,
     EnsureSizedNotStrError,
     EnsureStrError,
     EnsureTimeDeltaError,
     EnsureTimeError,
+    EnsureZonedDateTimeError,
     MaxNullableError,
     MinNullableError,
     apply_decorators,
@@ -49,18 +51,20 @@ from utilities.functions import (
     ensure_bytes,
     ensure_class,
     ensure_date,
-    ensure_datetime,
     ensure_float,
     ensure_hashable,
     ensure_int,
     ensure_member,
     ensure_not_none,
     ensure_number,
+    ensure_path,
+    ensure_plain_date_time,
     ensure_sized,
     ensure_sized_not_str,
     ensure_str,
     ensure_time,
-    ensure_timedelta,
+    ensure_time_delta,
+    ensure_zoned_date_time,
     first,
     get_class,
     get_class_name,
@@ -78,7 +82,6 @@ from utilities.functions import (
     is_sized,
     is_sized_not_str,
     is_string_mapping,
-    is_subclass_except_bool_int,
     is_tuple,
     is_tuple_or_str_mapping,
     make_isinstance,
@@ -87,24 +90,24 @@ from utilities.functions import (
     min_nullable,
     not_func,
     second,
+    to_bool,
     yield_object_attributes,
     yield_object_cached_properties,
     yield_object_properties,
 )
-from utilities.sentinel import sentinel
+from utilities.hypothesis import sentinels
+from utilities.sentinel import Sentinel, sentinel
+from utilities.whenever import NOW_UTC, ZERO_TIME, get_now, get_today
 
 if TYPE_CHECKING:
     import datetime as dt
     from collections.abc import Callable, Iterable
 
+    from whenever import PlainDateTime, TimeDelta, ZonedDateTime
+
     from utilities.types import Number
 
 
-_P = ParamSpec("_P")
-_R = TypeVar("_R")
-_T = TypeVar("_T")
-
-
 class TestApplyDecorators:
     @given(n=integers())
     def test_main(self, *, n: int) -> None:
@@ -113,9 +116,9 @@ class TestApplyDecorators:
         def negate(x: int, /) -> int:
             return -x
 
-        def increment(func: Callable[_P, _R], /) -> Callable[_P, _R]:
+        def increment[**P, T](func: Callable[P, T], /) -> Callable[P, T]:
             @wraps(func)
-            def wrapped(*args: _P.args, **kwargs: _P.kwargs) -> _R:
+            def wrapped(*args: P.args, **kwargs: P.kwargs) -> T:
                 nonlocal counter
                 counter += 1
                 return func(*args, **kwargs)
@@ -210,24 +213,6 @@ class TestEnsureDate:
             _ = ensure_date(sentinel, nullable=nullable)
 
 
-class TestEnsureDateTime:
-    @given(case=sampled_from([(get_now(), False), (get_now(), True), (None, True)]))
-    def test_main(self, *, case: tuple[dt.datetime | None, bool]) -> None:
-        obj, nullable = case
-        _ = ensure_datetime(obj, nullable=nullable)
-
-    @given(
-        case=sampled_from([
-            (False, "Object '.*' of type '.*' must be a datetime"),
-            (True, "Object '.*' of type '.*' must be a datetime or None"),
-        ])
-    )
-    def test_error(self, *, case: tuple[bool, str]) -> None:
-        nullable, match = case
-        with raises(EnsureDateTimeError, match=match):
-            _ = ensure_datetime(sentinel, nullable=nullable)
-
-
 class TestEnsureFloat:
     @given(case=sampled_from([(0.0, False), (0.0, True), (None, True)]))
     def test_main(self, *, case: tuple[float | None, bool]) -> None:
@@ -335,6 +320,48 @@ class TestEnsureNumber:
             _ = ensure_number(sentinel, nullable=nullable)
 
 
+class TestEnsurePath:
+    @given(case=sampled_from([(Path.home(), False), (Path.home(), True), (None, True)]))
+    def test_main(self, *, case: tuple[int | None, bool]) -> None:
+        obj, nullable = case
+        _ = ensure_path(obj, nullable=nullable)
+
+    @given(
+        case=sampled_from([
+            (False, "Object '.*' of type '.*' must be a Path"),
+            (True, "Object '.*' of type '.*' must be a Path or None"),
+        ])
+    )
+    def test_error(self, *, case: tuple[bool, str]) -> None:
+        nullable, match = case
+        with raises(EnsurePathError, match=match):
+            _ = ensure_path(sentinel, nullable=nullable)
+
+
+class TestEnsurePlainDateTime:
+    @given(
+        case=sampled_from([
+            (NOW_UTC.to_plain(), False),
+            (NOW_UTC.to_plain(), True),
+            (None, True),
+        ])
+    )
+    def test_main(self, *, case: tuple[PlainDateTime | None, bool]) -> None:
+        obj, nullable = case
+        _ = ensure_plain_date_time(obj, nullable=nullable)
+
+    @given(
+        case=sampled_from([
+            (False, "Object '.*' of type '.*' must be a plain date-time"),
+            (True, "Object '.*' of type '.*' must be a plain date-time or None"),
+        ])
+    )
+    def test_error(self, *, case: tuple[bool, str]) -> None:
+        nullable, match = case
+        with raises(EnsurePlainDateTimeError, match=match):
+            _ = ensure_plain_date_time(sentinel, nullable=nullable)
+
+
 class TestEnsureSized:
     @given(obj=sampled_from([[], (), ""]))
     def test_main(self, *, obj: Any) -> None:
@@ -403,20 +430,38 @@ class TestEnsureTime:
 
 class TestEnsureTimeDelta:
     @given(case=sampled_from([(ZERO_TIME, False), (ZERO_TIME, True), (None, True)]))
-    def test_main(self, *, case: tuple[dt.timedelta | None, bool]) -> None:
+    def test_main(self, *, case: tuple[TimeDelta | None, bool]) -> None:
         obj, nullable = case
-        _ = ensure_timedelta(obj, nullable=nullable)
+        _ = ensure_time_delta(obj, nullable=nullable)
 
     @given(
         case=sampled_from([
-            (False, "Object '.*' of type '.*' must be a timedelta"),
-            (True, "Object '.*' of type '.*' must be a timedelta or None"),
+            (False, "Object '.*' of type '.*' must be a time-delta"),
+            (True, "Object '.*' of type '.*' must be a time-delta or None"),
         ])
     )
     def test_error(self, *, case: tuple[bool, str]) -> None:
         nullable, match = case
         with raises(EnsureTimeDeltaError, match=match):
-            _ = ensure_timedelta(sentinel, nullable=nullable)
+            _ = ensure_time_delta(sentinel, nullable=nullable)
+
+
+class TestEnsureZonedDateTime:
+    @given(case=sampled_from([(NOW_UTC, False), (NOW_UTC, True), (None, True)]))
+    def test_main(self, *, case: tuple[ZonedDateTime | None, bool]) -> None:
+        obj, nullable = case
+        _ = ensure_zoned_date_time(obj, nullable=nullable)
+
+    @given(
+        case=sampled_from([
+            (False, "Object '.*' of type '.*' must be a zoned date-time"),
+            (True, "Object '.*' of type '.*' must be a zoned date-time or None"),
+        ])
+    )
+    def test_error(self, *, case: tuple[bool, str]) -> None:
+        nullable, match = case
+        with raises(EnsureZonedDateTimeError, match=match):
+            _ = ensure_zoned_date_time(sentinel, nullable=nullable)
 
 
 class TestFirst:
@@ -490,7 +535,7 @@ class TestGetFuncNameAndGetFuncQualName:
 
     def test_decorated(self) -> None:
         @wraps(identity)
-        def wrapped(x: _T, /) -> _T:
+        def wrapped[T](x: T, /) -> T:
             return identity(x)
 
         assert get_func_name(wrapped) == "identity"
@@ -509,7 +554,7 @@ class TestGetFuncNameAndGetFuncQualName:
 
     def test_object(self) -> None:
         class Example:
-            def __call__(self, x: _T, /) -> _T:
+            def __call__[T](self, x: T, /) -> T:
                 return identity(x)
 
         obj = Example()
@@ -518,7 +563,7 @@ class TestGetFuncNameAndGetFuncQualName:
 
     def test_obj_method(self) -> None:
         class Example:
-            def obj_method(self, x: _T) -> _T:
+            def obj_method[T](self, x: T) -> T:
                 return identity(x)
 
         obj = Example()
@@ -531,7 +576,7 @@ class TestGetFuncNameAndGetFuncQualName:
     def test_obj_classmethod(self) -> None:
         class Example:
             @classmethod
-            def obj_classmethod(cls: _T) -> _T:
+            def obj_classmethod[T](cls: T) -> T:
                 return identity(cls)
 
         assert get_func_name(Example.obj_classmethod) == "Example.obj_classmethod"
@@ -543,7 +588,7 @@ class TestGetFuncNameAndGetFuncQualName:
     def test_obj_staticmethod(self) -> None:
         class Example:
             @staticmethod
-            def obj_staticmethod(x: _T) -> _T:
+            def obj_staticmethod[T](x: T) -> T:
                 return identity(x)
 
         assert get_func_name(Example.obj_staticmethod) == "Example.obj_staticmethod"
@@ -719,20 +764,6 @@ class TestIsStringMapping:
         assert result is expected
 
 
-class TestIsSubclassExceptBoolInt:
-    @given(
-        case=sampled_from([(bool, bool, True), (bool, int, False), (int, int, True)])
-    )
-    def test_main(self, *, case: tuple[type[Any], type[Any], bool]) -> None:
-        x, y, expected = case
-        assert is_subclass_except_bool_int(x, y) is expected
-
-    def test_subclass_of_int(self) -> None:
-        class MyInt(int): ...
-
-        assert not is_subclass_except_bool_int(bool, MyInt)
-
-
 class TestIsTuple:
     @given(case=sampled_from([(None, False), ((1, 2, 3), True), ([1, 2, 3], False)]))
     def test_main(self, *, case: tuple[Any, bool]) -> None:
@@ -886,6 +917,16 @@ class TestSecond:
         assert second(pair) == y
 
 
+class TestToBool:
+    @given(bool_=booleans() | none() | sentinels())
+    def test_bool_none_or_sentinel(self, *, bool_: bool | None | Sentinel) -> None:
+        assert to_bool(bool_=bool_) is bool_
+
+    @given(bool_=booleans())
+    def test_callable(self, *, bool_: bool) -> None:
+        assert to_bool(bool_=lambda: bool_) is bool_
+
+
 class TestYieldObjectAttributes:
     @given(n=integers())
     def test_main(self, *, n: int) -> None:
