@@ -1,13 +1,11 @@
 from __future__ import annotations
 
-import datetime as dt
 from itertools import pairwise
 from pathlib import Path
 from re import search
-from subprocess import PIPE, check_output
 from typing import TYPE_CHECKING, Any, cast
 
-from hypothesis import HealthCheck, Phase, assume, given, settings
+from hypothesis import Phase, assume, given, settings
 from hypothesis.errors import InvalidArgument
 from hypothesis.extra.numpy import array_shapes
 from hypothesis.strategies import (
@@ -16,76 +14,68 @@ from hypothesis.strategies import (
     booleans,
     composite,
     data,
-    datetimes,
     floats,
     integers,
     just,
     none,
     sets,
-    timedeltas,
     timezones,
-    uuids,
 )
+from libcst import Import, ImportFrom
 from luigi import Task
 from numpy import inf, int64, isfinite, isinf, isnan, ravel, rint
 from pathvalidate import validate_filepath
-from pytest import raises
-from sqlalchemy import Column, Integer, MetaData, Table, insert, select
-from sqlalchemy.ext.asyncio import AsyncEngine
-
-from tests.conftest import SKIPIF_CI_AND_NOT_LINUX, SKIPIF_CI_AND_WINDOWS
-from utilities.datetime import (
-    MAX_DATE_TWO_DIGIT_YEAR,
-    MIN_DATE_TWO_DIGIT_YEAR,
-    MINUTE,
-    date_duration_to_timedelta,
-    datetime_duration_to_float,
-    datetime_duration_to_timedelta,
-    is_integral_timedelta,
-    parse_two_digit_year,
+from pytest import mark, raises
+from whenever import (
+    Date,
+    DateDelta,
+    DateTimeDelta,
+    MonthDay,
+    PlainDateTime,
+    Time,
+    TimeDelta,
+    TimeZoneNotFoundError,
+    YearMonth,
+    ZonedDateTime,
 )
+
 from utilities.functions import ensure_int
-from utilities.git import _GIT_REMOTE_GET_URL_ORIGIN, _GIT_REV_PARSE_ABBREV_REV_HEAD
 from utilities.hypothesis import (
-    _SQLALCHEMY_ENGINE_DIALECTS,
-    LocalDateTimesError,
-    MaybeSearchStrategy,
     Shape,
-    ZonedDateTimesError,
     _Draw2DefaultGeneratedSentinelError,
     _Draw2InputResolvedToSentinelError,
+    _freq_units,
     assume_does_not_raise,
     bool_arrays,
-    date_durations,
-    dates_two_digit_year,
-    datetime_durations,
+    date_deltas,
+    date_time_deltas,
+    dates,
     draw2,
     float32s,
     float64s,
     float_arrays,
     floats_extra,
+    freqs,
     git_repos,
     hashables,
+    import_froms,
+    imports,
     int32s,
     int64s,
     int_arrays,
     lists_fixed_length,
-    local_datetimes,
-    min_and_max_datetimes,
-    min_and_maybe_max_datetimes,
-    min_and_maybe_max_sizes,
-    months,
+    month_days,
     namespace_mixins,
     numbers,
     pairs,
     paths,
+    plain_datetimes,
     random_states,
     sentinels,
     sets_fixed_length,
     settings_with_reduced_examples,
     setup_hypothesis_profiles,
     slices,
-    sqlalchemy_engines,
     str_arrays,
     temp_dirs,
     temp_paths,
@@ -95,14 +85,17 @@ from utilities.hypothesis import (
     text_clean,
     text_digits,
     text_printable,
-    timedeltas_2w,
+    time_deltas,
+    times,
     triples,
     uint32s,
     uint64s,
     versions,
-    yield_test_redis,
+    year_months,
     zoned_datetimes,
 )
+from utilities.iterables import one
+from utilities.libcst import parse_import
 from utilities.math import (
     MAX_FLOAT32,
     MAX_FLOAT64,
@@ -116,33 +109,26 @@ from utilities.math import (
     MIN_INT64,
     MIN_UINT32,
     MIN_UINT64,
-    is_at_least,
-    is_at_most,
 )
 from utilities.os import temp_environ
 from utilities.platform import maybe_yield_lower_case
 from utilities.sentinel import Sentinel
-from utilities.sqlalchemy import Dialect, _get_dialect
 from utilities.version import Version
 from utilities.whenever import (
-    MAX_SERIALIZABLE_TIMEDELTA,
-    MIN_SERIALIZABLE_TIMEDELTA,
-    check_valid_zoned_datetime,
-    parse_duration,
-    parse_timedelta,
-    serialize_duration,
-    serialize_timedelta,
+    DATE_TWO_DIGIT_YEAR_MAX,
+    DATE_TWO_DIGIT_YEAR_MIN,
+    Freq,
+    to_days,
+    to_nanos,
 )
 
 if TYPE_CHECKING:
     from collections.abc import Iterable
     from collections.abc import Set as AbstractSet
-    from uuid import UUID
     from zoneinfo import ZoneInfo
 
-    from utilities.datetime import Month
     from utilities.tempfile import TemporaryDirectory
-    from utilities.types import Number
+    from utilities.types import DateTimeRoundUnit, Number
 
 
 class TestAssumeDoesNotRaise:
@@ -188,140 +174,63 @@ class TestBoolArrays:
         assert array.shape == shape
 
 
-class TestDateDurations:
-    @given(
-        data=data(),
-        min_int=integers() | none(),
-        max_int=integers() | none(),
-        min_timedelta=timedeltas() | none(),
-        max_timedelta=timedeltas() | none(),
-    )
-    def test_main(
-        self,
-        *,
-        data: DataObject,
-        min_int: int | None,
-        max_int: int | None,
-        min_timedelta: dt.timedelta | None,
-        max_timedelta: dt.timedelta | None,
-    ) -> None:
-        duration = data.draw(
-            date_durations(
-                min_int=min_int,
-                max_int=max_int,
-                min_timedelta=min_timedelta,
-                max_timedelta=max_timedelta,
+class TestDateDeltas:
+    @given(data=data(), parsable=booleans())
+    def test_main(self, *, data: DataObject, parsable: bool) -> None:
+        min_value = data.draw(date_deltas() | none())
+        max_value = data.draw(date_deltas() | none())
+        with assume_does_not_raise(InvalidArgument):
+            delta = data.draw(
+                date_deltas(min_value=min_value, max_value=max_value, parsable=parsable)
             )
-        )
-        assert isinstance(duration, int | float | dt.timedelta)
-        match duration:
-            case int():
-                if min_int is not None:
-                    assert duration >= min_int
-                if max_int is not None:
-                    assert duration <= max_int
-                if min_timedelta is not None:
-                    assert date_duration_to_timedelta(duration) >= min_timedelta
-                if max_timedelta is not None:
-                    assert date_duration_to_timedelta(duration) <= max_timedelta
-            case float():
-                assert duration == round(duration)
-                if min_int is not None:
-                    assert duration >= min_int
-                if max_int is not None:
-                    assert duration <= max_int
-                if min_timedelta is not None:
-                    assert date_duration_to_timedelta(duration) >= min_timedelta
-                if max_timedelta is not None:
-                    assert date_duration_to_timedelta(duration) <= max_timedelta
-            case dt.timedelta():
-                assert is_integral_timedelta(duration)
-                if min_int is not None:
-                    assert duration >= date_duration_to_timedelta(min_int)
-                if max_int is not None:
-                    assert duration <= date_duration_to_timedelta(max_int)
-                if min_timedelta is not None:
-                    assert duration >= min_timedelta
-                if max_timedelta is not None:
-                    assert duration <= max_timedelta
-
-    @given(data=data())
-    def test_two_way(self, *, data: DataObject) -> None:
-        duration = data.draw(date_durations(two_way=True))
-        ser = serialize_duration(duration)
-        _ = parse_duration(ser)
+        assert isinstance(delta, DateDelta)
+        days = to_days(delta)
+        if min_value is not None:
+            assert days >= to_days(min_value)
+        if max_value is not None:
+            assert days <= to_days(max_value)
+        if parsable:
+            assert DateDelta.parse_common_iso(delta.format_common_iso()) == delta
 
 
-class TestDatesTwoDigitYear:
-    @given(data=data())
-    def test_main(self, *, data: DataObject) -> None:
-        min_value, max_value = data.draw(pairs(dates_two_digit_year(), sorted=True))
-        date = data.draw(dates_two_digit_year(min_value=min_value, max_value=max_value))
-        assert (
-            max(min_value, MIN_DATE_TWO_DIGIT_YEAR)
-            <= date
-            <= min(max_value, MAX_DATE_TWO_DIGIT_YEAR)
-        )
-        year = f"{date:%y}"
-        parsed = parse_two_digit_year(year)
-        assert date.year == parsed
-
-
-class TestDateTimeDurations:
-    @given(
-        data=data(),
-        min_number=numbers() | none(),
-        max_number=numbers() | none(),
-        min_timedelta=timedeltas() | none(),
-        max_timedelta=timedeltas() | none(),
-    )
-    def test_main(
-        self,
-        *,
-        data: DataObject,
-        min_number: Number | None,
-        max_number: Number | None,
-        min_timedelta: dt.timedelta | None,
-        max_timedelta: dt.timedelta | None,
-    ) -> None:
-        duration = data.draw(
-            datetime_durations(
-                min_number=min_number,
-                max_number=max_number,
-                min_timedelta=min_timedelta,
-                max_timedelta=max_timedelta,
+class TestDateTimeDeltas:
+    @given(data=data(), parsable=booleans())
+    def test_main(self, *, data: DataObject, parsable: bool) -> None:
+        min_value = data.draw(date_time_deltas() | none())
+        max_value = data.draw(date_time_deltas() | none())
+        with assume_does_not_raise(InvalidArgument):
+            delta = data.draw(
+                date_time_deltas(
+                    min_value=min_value, max_value=max_value, parsable=parsable
+                )
             )
-        )
-        assert isinstance(duration, int | float | dt.timedelta)
-        match duration:
-            case int() | float():
-                if min_number is not None:
-                    assert is_at_least(duration, min_number, abs_tol=1e-6)
-                if max_number is not None:
-                    assert is_at_most(duration, max_number, abs_tol=1e-6)
-                if min_timedelta is not None:
-                    assert is_at_least(
-                        duration, datetime_duration_to_float(min_timedelta)
-                    )
-                if max_timedelta is not None:
-                    assert is_at_most(
-                        duration, datetime_duration_to_float(max_timedelta)
-                    )
-            case dt.timedelta():
-                if min_number is not None:
-                    assert duration >= datetime_duration_to_timedelta(min_number)
-                if max_number is not None:
-                    assert duration <= datetime_duration_to_timedelta(max_number)
-                if min_timedelta is not None:
-                    assert duration >= min_timedelta
-                if max_timedelta is not None:
-                    assert duration <= max_timedelta
+        assert isinstance(delta, DateTimeDelta)
+        nanos = to_nanos(delta)
+        if min_value is not None:
+            assert nanos >= to_nanos(min_value)
+        if max_value is not None:
+            assert nanos <= to_nanos(max_value)
+        if parsable:
+            assert DateTimeDelta.parse_common_iso(delta.format_common_iso()) == delta
 
-    @given(data=data())
-    def test_two_way(self, *, data: DataObject) -> None:
-        duration = data.draw(datetime_durations(two_way=True))
-        ser = serialize_duration(duration)
-        _ = parse_duration(ser)
+
+class TestDates:
+    @given(data=data(), two_digit=booleans())
+    def test_main(self, *, data: DataObject, two_digit: bool) -> None:
+        min_value = data.draw(dates() | none())
+        max_value = data.draw(dates() | none())
+        with assume_does_not_raise(InvalidArgument):
+            date = data.draw(
+                dates(min_value=min_value, max_value=max_value, two_digit=two_digit)
+            )
+        assert isinstance(date, Date)
+        assert Date.parse_common_iso(date.format_common_iso()) == date
+        if min_value is not None:
+            assert date >= min_value
+        if max_value is not None:
+            assert date <= max_value
+        if two_digit:
+            assert DATE_TWO_DIGIT_YEAR_MIN <= date <= DATE_TWO_DIGIT_YEAR_MAX
 
 
 class TestDraw2:
@@ -587,25 +496,22 @@ class TestFloatsExtra:
         assert x == round(x)
 
 
+class TestFreqs:
+    @given(data=data(), unit=_freq_units() | none())
+    def test_main(self, *, data: DataObject, unit: DateTimeRoundUnit | None) -> None:
+        freq = data.draw(freqs(unit=unit))
+        assert isinstance(freq, Freq)
+        if unit is not None:
+            assert freq.unit == unit
+
+
 class TestGitRepos:
     @given(data=data())
     @settings_with_reduced_examples()
     def test_main(self, *, data: DataObject) -> None:
-        branch = data.draw(text_ascii(min_size=1) | none())
-        remote = data.draw(text_ascii(min_size=1) | none())
-        root = data.draw(git_repos(branch=branch, remote=remote))
+        root = data.draw(git_repos())
         files = set(root.iterdir())
         assert Path(root, ".git") in files
-        if branch is not None:
-            output = check_output(
-                _GIT_REV_PARSE_ABBREV_REV_HEAD, stderr=PIPE, cwd=root, text=True
-            )
-            assert output.strip("\n") == branch
-        if remote is not None:
-            output = check_output(
-                _GIT_REMOTE_GET_URL_ORIGIN, stderr=PIPE, cwd=root, text=True
-            )
-            assert output.strip("\n") == remote
 
 
 class TestHashables:
@@ -615,6 +521,36 @@ class TestHashables:
         _ = hash(x)
 
 
+class TestImports:
+    @given(data=data())
+    def test_main(self, *, data: DataObject) -> None:
+        min_depth = data.draw(integers(0, 10) | none())
+        max_depth = data.draw(integers(0, 10) | none())
+        with assume_does_not_raise(InvalidArgument):
+            imp = data.draw(imports(min_depth=min_depth, max_depth=max_depth))
+        assert isinstance(imp, Import)
+        parsed = one(parse_import(imp))
+        if min_depth is not None:
+            assert len(parsed.module.split(".")) >= min_depth
+        if max_depth is not None:
+            assert len(parsed.module.split(".")) <= max_depth
+
+
+class TestImportFroms:
+    @given(data=data())
+    def test_main(self, *, data: DataObject) -> None:
+        min_depth = data.draw(integers(0, 10) | none())
+        max_depth = data.draw(integers(0, 10) | none())
+        with assume_does_not_raise(InvalidArgument):
+            imp = data.draw(import_froms(min_depth=min_depth, max_depth=max_depth))
+        assert isinstance(imp, ImportFrom)
+        parsed = one(parse_import(imp))
+        if min_depth is not None:
+            assert len(parsed.module.split(".")) >= min_depth
+        if max_depth is not None:
+            assert len(parsed.module.split(".")) <= max_depth
+
+
 class TestIntArrays:
     @given(
         data=data(),
@@ -677,131 +613,19 @@ class TestListsFixedLength:
             assert sorted(result) == result
 
 
-class TestLocalDateTimes:
-    @given(data=data(), min_value=datetimes(), max_value=datetimes())
-    @settings(suppress_health_check={HealthCheck.filter_too_much})
-    def test_main(
-        self, *, data: DataObject, min_value: dt.datetime, max_value: dt.datetime
-    ) -> None:
-        with assume_does_not_raise(InvalidArgument):
-            datetime = data.draw(
-                local_datetimes(min_value=min_value, max_value=max_value)
-            )
-        assert datetime.tzinfo is None
-        assert min_value <= datetime <= max_value
-
-    @given(data=data())
-    def test_rounding(self, *, data: DataObject) -> None:
-        min_value, max_value = data.draw(pairs(local_datetimes(), sorted=True))
-        datetime = data.draw(
-            local_datetimes(
-                min_value=min_value,
-                max_value=max_value,
-                round_="standard",
-                timedelta=MINUTE,
-            )
-        )
-        assert isinstance(datetime, dt.datetime)
-        assert datetime.second == datetime.microsecond == 0
-        assert min_value <= datetime <= max_value
-
-    @given(data=data())
-    def test_error_rounding(self, *, data: DataObject) -> None:
-        with raises(
-            LocalDateTimesError, match="Rounding requires a timedelta; got None"
-        ):
-            _ = data.draw(local_datetimes(round_="standard"))
-
-
-class TestMinAndMaxDateTimes:
-    @given(
-        data=data(),
-        min_value=zoned_datetimes() | none() | just(zoned_datetimes() | none()),
-        max_value=zoned_datetimes() | none() | just(zoned_datetimes() | none()),
-    )
-    def test_main(
-        self,
-        *,
-        data: DataObject,
-        min_value: MaybeSearchStrategy[dt.datetime | None],
-        max_value: MaybeSearchStrategy[dt.datetime | None],
-    ) -> None:
-        min_datetime, max_datetime = data.draw(
-            min_and_max_datetimes(min_value=min_value, max_value=max_value)
-        )
-        assert min_datetime <= max_datetime
-        if isinstance(min_value, dt.datetime):
-            assert min_datetime == min_value
-        if isinstance(max_value, dt.datetime):
-            assert max_datetime == max_value
-
-
-class TestMinAndMaybeMaxDateTimes:
-    @given(
-        data=data(),
-        min_value=zoned_datetimes() | none() | just(zoned_datetimes() | none()),
-        max_value=zoned_datetimes()
-        | none()
-        | sentinels()
-        | just(zoned_datetimes() | none() | sentinels()),
-    )
-    def test_main(
-        self,
-        *,
-        data: DataObject,
-        min_value: MaybeSearchStrategy[dt.datetime | None],
-        max_value: MaybeSearchStrategy[dt.datetime | None | Sentinel],
-    ) -> None:
-        min_datetime, max_datetime = data.draw(
-            min_and_maybe_max_datetimes(min_value=min_value, max_value=max_value)
-        )
-        assert (max_datetime is None) or (min_datetime <= max_datetime)
-        if isinstance(min_value, dt.datetime):
-            assert min_datetime == min_value
-        if isinstance(max_value, dt.datetime) or (max_value is None):
-            assert max_datetime == max_value
-
-
-class TestMinAndMaybeMaxSizes:
-    @given(
-        data=data(),
-        min_value=integers(min_value=0) | none() | just(integers(min_value=0) | none()),
-        max_value=integers(min_value=0)
-        | none()
-        | sentinels()
-        | just(integers(min_value=0) | none() | sentinels()),
-    )
-    def test_main(
-        self,
-        *,
-        data: DataObject,
-        min_value: MaybeSearchStrategy[int | None],
-        max_value: MaybeSearchStrategy[int | None | Sentinel],
-    ) -> None:
-        min_size, max_size = data.draw(
-            min_and_maybe_max_sizes(min_value=min_value, max_value=max_value)
-        )
-        assert min_size >= 0
-        assert (max_size is None) or (min_size <= max_size)
-        if isinstance(min_value, int):
-            assert min_size == min_value
-        if isinstance(max_value, int) or (max_value is None):
-            assert max_size == max_value
-
-
-class TestMonths:
+class TestMonthDays:
     @given(data=data())
     def test_main(self, *, data: DataObject) -> None:
-        _ = data.draw(months())
-
-    @given(data=data(), min_value=months(), max_value=months())
-    @settings(suppress_health_check={HealthCheck.filter_too_much})
-    def test_min_and_max_value(
-        self, *, data: DataObject, min_value: Month, max_value: Month
-    ) -> None:
-        _ = assume(min_value <= max_value)
-        month = data.draw(months(min_value=min_value, max_value=max_value))
-        assert min_value <= month <= max_value
+        min_value = data.draw(month_days() | none())
+        max_value = data.draw(month_days() | none())
+        with assume_does_not_raise(InvalidArgument):
+            month_day = data.draw(month_days(min_value=min_value, max_value=max_value))
+        assert isinstance(month_day, MonthDay)
+        assert MonthDay.parse_common_iso(month_day.format_common_iso()) == month_day
+        if min_value is not None:
+            assert month_day >= min_value
+        if max_value is not None:
+            assert month_day <= max_value
 
 
 class TestNamespaceMixins:
@@ -856,10 +680,34 @@ class TestPairs:
 class TestPaths:
     @given(data=data())
     def test_main(self, *, data: DataObject) -> None:
-        path = data.draw(paths())
+        min_depth = data.draw(integers(0, 5) | none())
+        max_depth = data.draw(integers(0, 5) | none())
+        with assume_does_not_raise(InvalidArgument):
+            path = data.draw(paths(min_depth=min_depth, max_depth=max_depth))
         assert isinstance(path, Path)
         assert not path.is_absolute()
         validate_filepath(str(path))
+        if min_depth is not None:
+            assert len(path.parts) >= min_depth
+        if max_depth is not None:
+            assert len(path.parts) <= max_depth
+
+
+class TestPlainDateTimes:
+    @given(data=data())
+    def test_main(self, *, data: DataObject) -> None:
+        min_value = data.draw(plain_datetimes() | none())
+        max_value = data.draw(plain_datetimes() | none())
+        with assume_does_not_raise(InvalidArgument):
+            datetime = data.draw(
+                plain_datetimes(min_value=min_value, max_value=max_value)
+            )
+        assert isinstance(datetime, PlainDateTime)
+        assert PlainDateTime.parse_common_iso(datetime.format_common_iso()) == datetime
+        if min_value is not None:
+            assert datetime >= min_value
+        if max_value is not None:
+            assert datetime <= max_value
 
 
 class TestRandomStates:
@@ -932,38 +780,6 @@ class TestSlices:
             _ = data.draw(slices(iter_len, slice_len=iter_len + 1))
 
 
-class TestSQLAlchemyEngines:
-    @given(
-        data=data(),
-        name=uuids(),
-        dialect=_SQLALCHEMY_ENGINE_DIALECTS,
-        ids=sets(integers(0, 10), min_size=1),
-    )
-    @settings(phases={Phase.generate})
-    async def test_main(
-        self, *, data: DataObject, name: UUID, dialect: Dialect, ids: set[int]
-    ) -> None:
-        table = Table(
-            f"test_{name}", MetaData(), Column("id_", Integer, primary_key=True)
-        )
-        engine = await sqlalchemy_engines(data, table, dialect=dialect)
-        assert isinstance(engine, AsyncEngine)
-        assert _get_dialect(engine) == dialect
-        if dialect == "sqlite":
-            database = engine.url.database
-            assert database is not None
-            assert not Path(database).exists()
-        async with engine.begin() as conn:
-            await conn.run_sync(table.metadata.create_all)
-        ins = insert(table).values([(id_,) for id_ in ids])
-        async with engine.begin() as conn:
-            _ = await conn.execute(ins)
-        sel = select(table.c["id_"])
-        async with engine.begin() as conn:
-            results = (await conn.execute(sel)).scalars().all()
-        assert set(results) == ids
-
-
 class TestStrArrays:
     @given(
         data=data(),
@@ -1033,6 +849,7 @@ class TestTempPaths:
         assert len(set(path.iterdir())) == 0
 
     @given(path=temp_paths(), contents=sets(text_ascii(min_size=1), max_size=10))
+    @mark.flaky
     def test_writing_files(self, *, path: Path, contents: AbstractSet[str]) -> None:
         assert len(set(path.iterdir())) == 0
         as_set = set(maybe_yield_lower_case(contents))
@@ -1119,25 +936,34 @@ class TestTextPrintable:
             assert len(text) <= max_size
 
 
-class TestTimeDeltas2W:
-    @given(
-        data=data(),
-        min_value=timedeltas(
-            min_value=MIN_SERIALIZABLE_TIMEDELTA, max_value=MAX_SERIALIZABLE_TIMEDELTA
-        ),
-        max_value=timedeltas(
-            min_value=MIN_SERIALIZABLE_TIMEDELTA, max_value=MAX_SERIALIZABLE_TIMEDELTA
-        ),
-    )
-    @settings(suppress_health_check={HealthCheck.filter_too_much})
-    def test_main(
-        self, *, data: DataObject, min_value: dt.timedelta, max_value: dt.timedelta
-    ) -> None:
-        _ = assume(min_value <= max_value)
-        timedelta = data.draw(timedeltas_2w(min_value=min_value, max_value=max_value))
-        ser = serialize_timedelta(timedelta)
-        _ = parse_timedelta(ser)
-        assert min_value <= timedelta <= max_value
+class TestTimeDeltas:
+    @given(data=data())
+    def test_main(self, *, data: DataObject) -> None:
+        min_value = data.draw(time_deltas() | none())
+        max_value = data.draw(time_deltas() | none())
+        with assume_does_not_raise(InvalidArgument):
+            delta = data.draw(time_deltas(min_value=min_value, max_value=max_value))
+        assert isinstance(delta, TimeDelta)
+        assert TimeDelta.parse_common_iso(delta.format_common_iso()) == delta
+        if min_value is not None:
+            assert delta >= min_value
+        if max_value is not None:
+            assert delta <= max_value
+
+
+class TestTimes:
+    @given(data=data())
+    def test_main(self, *, data: DataObject) -> None:
+        min_value = data.draw(times() | none())
+        max_value = data.draw(times() | none())
+        with assume_does_not_raise(InvalidArgument):
+            time = data.draw(times(min_value=min_value, max_value=max_value))
+        assert isinstance(time, Time)
+        assert Time.parse_common_iso(time.format_common_iso()) == time
+        if min_value is not None:
+            assert time >= min_value
+        if max_value is not None:
+            assert time <= max_value
 
 
 class TestTriples:
@@ -1181,85 +1007,51 @@ class TestVersions:
             assert version.suffix is None
 
 
-@SKIPIF_CI_AND_NOT_LINUX
-class TestYieldTestRedis:
-    @given(data=data(), value=int32s())
-    @settings_with_reduced_examples()
-    async def test_core(self, *, data: DataObject, value: int) -> None:
-        async with yield_test_redis(data) as test:
-            assert not await test.redis.exists(test.key)
-            _ = await test.redis.set(test.key, value)
-            result = int(await test.redis.get(test.key))
-            assert result == value
+class TestYearMonths:
+    @given(data=data(), two_digit=booleans())
+    def test_main(self, *, data: DataObject, two_digit: bool) -> None:
+        min_value = data.draw(year_months() | none())
+        max_value = data.draw(year_months() | none())
+        with assume_does_not_raise(InvalidArgument):
+            year_month = data.draw(
+                year_months(
+                    min_value=min_value, max_value=max_value, two_digit=two_digit
+                )
+            )
+        assert isinstance(year_month, YearMonth)
+        assert YearMonth.parse_common_iso(year_month.format_common_iso()) == year_month
+        if min_value is not None:
+            assert year_month >= min_value
+        if max_value is not None:
+            assert year_month <= max_value
+        if two_digit:
+            assert year_month.on_day(1) >= DATE_TWO_DIGIT_YEAR_MIN
+            assert year_month.on_day(28) <= DATE_TWO_DIGIT_YEAR_MAX
 
 
 class TestZonedDateTimes:
-    @given(
-        data=data(),
-        min_value=datetimes(timezones=timezones() | just(dt.UTC) | none()),
-        max_value=datetimes(timezones=timezones() | just(dt.UTC) | none()),
-        time_zone=timezones() | just(dt.UTC),
-        time_zone_extra=timezones() | just(dt.UTC),
-    )
-    @settings(suppress_health_check={HealthCheck.filter_too_much})
-    def test_main(
-        self,
-        *,
-        data: DataObject,
-        min_value: dt.datetime,
-        max_value: dt.datetime,
-        time_zone: ZoneInfo,
-        time_zone_extra: ZoneInfo,
-    ) -> None:
+    @given(data=data(), time_zone=timezones())
+    def test_main(self, *, data: DataObject, time_zone: ZoneInfo) -> None:
+        min_value = data.draw(zoned_datetimes() | none())
+        max_value = data.draw(zoned_datetimes() | none())
         with assume_does_not_raise(InvalidArgument):
             datetime = data.draw(
                 zoned_datetimes(
                     min_value=min_value, max_value=max_value, time_zone=time_zone
                 )
             )
-        assert datetime.tzinfo is time_zone
-        if min_value.tzinfo is None:
-            min_value_ = min_value.replace(tzinfo=time_zone)
-        else:
-            min_value_ = min_value.astimezone(time_zone)
-        if max_value.tzinfo is None:
-            max_value_ = max_value.replace(tzinfo=time_zone)
-        else:
-            max_value_ = max_value.astimezone(time_zone)
-        assert min_value_ <= datetime <= max_value_
-        _ = datetime.astimezone(time_zone_extra)
+        assert isinstance(datetime, ZonedDateTime)
+        _ = datetime.py_datetime()
+        assert ZonedDateTime.parse_common_iso(datetime.format_common_iso()) == datetime
+        assert datetime.tz == time_zone.key
+        if min_value is not None:
+            assert datetime >= min_value
+        if max_value is not None:
+            assert datetime <= max_value
 
-    @given(data=data())
-    def test_rounding(self, *, data: DataObject) -> None:
-        min_value, max_value = data.draw(pairs(zoned_datetimes(), sorted=True))
-        datetime = data.draw(
-            zoned_datetimes(
-                min_value=min_value,
-                max_value=max_value,
-                round_="standard",
-                timedelta=MINUTE,
-            )
-        )
-        assert isinstance(datetime, dt.datetime)
-        assert datetime.second == datetime.microsecond == 0
-        assert min_value <= datetime <= max_value
-
-    @given(
-        data=data(),
-        min_value=zoned_datetimes(valid=True),
-        max_value=zoned_datetimes(valid=True),
-    )
-    @SKIPIF_CI_AND_WINDOWS
-    def test_valid(
-        self, *, data: DataObject, min_value: dt.datetime, max_value: dt.datetime
-    ) -> None:
-        _ = assume(min_value <= max_value)
-        datetime = data.draw(zoned_datetimes(valid=True))
-        check_valid_zoned_datetime(datetime)
-
-    @given(data=data())
-    def test_error_rounding(self, *, data: DataObject) -> None:
-        with raises(
-            ZonedDateTimesError, match="Rounding requires a timedelta; got None"
-        ):
-            _ = data.draw(zoned_datetimes(round_="standard"))
+    @given(data=data(), time_zone=timezones())
+    def test_examples(self, *, data: DataObject, time_zone: ZoneInfo) -> None:
+        with assume_does_not_raise(TimeZoneNotFoundError):
+            max_value = ZonedDateTime(1, 1, 2, tz=time_zone.key)
+        datetime = data.draw(zoned_datetimes(max_value=max_value, time_zone=time_zone))
+        _ = datetime.py_datetime()
