@@ -1,13 +1,14 @@
 from __future__ import annotations
 
 import re
-from dataclasses import dataclass, replace
+from dataclasses import dataclass
 from enum import Enum, auto
+from functools import cmp_to_key
 from itertools import chain, repeat
-from math import isfinite, isinf, isnan
+from math import isfinite, isinf, isnan, nan
 from operator import add, neg, sub
 from re import DOTALL
-from typing import TYPE_CHECKING, Any, ClassVar, override
+from typing import TYPE_CHECKING, Any, ClassVar
 
 from hypothesis import given
 from hypothesis.strategies import (
@@ -15,7 +16,6 @@ from hypothesis.strategies import (
     binary,
     booleans,
     data,
-    datetimes,
     dictionaries,
     floats,
     frozensets,
@@ -31,14 +31,9 @@ from hypothesis.strategies import (
 )
 from pytest import mark, param, raises
 
-from tests.test_operator import make_objects
+from tests.test_objects.objects import objects
 from utilities.functions import is_sequence_of
-from utilities.hypothesis import (
-    sentinels,
-    sets_fixed_length,
-    text_ascii,
-    zoned_datetimes,
-)
+from utilities.hypothesis import pairs, sentinels, sets_fixed_length, text_ascii
 from utilities.iterables import (
     CheckBijectionError,
     CheckDuplicatesError,
@@ -51,7 +46,6 @@ from utilities.iterables import (
     CheckSubSetError,
     CheckSuperMappingError,
     CheckSuperSetError,
-    Collection,
     EnsureIterableError,
     EnsureIterableNotStrError,
     MergeStrMappingsError,
@@ -59,6 +53,8 @@ from utilities.iterables import (
     OneMaybeEmptyError,
     OneMaybeNonUniqueError,
     OneNonUniqueError,
+    OneStrEmptyError,
+    OneStrNonUniqueError,
     OneUniqueEmptyError,
     OneUniqueNonUniqueError,
     ResolveIncludeAndExcludeError,
@@ -67,9 +63,9 @@ from utilities.iterables import (
     _ApplyBijectionDuplicateValuesError,
     _CheckUniqueModuloCaseDuplicateLowerCaseStringsError,
     _CheckUniqueModuloCaseDuplicateStringsError,
-    _OneStrEmptyError,
-    _OneStrNonUniqueError,
-    _sort_iterable_cmp_datetimes,
+    _RangePartitionsNumError,
+    _RangePartitionsStopError,
+    _RangePartitionsTotalError,
     _sort_iterable_cmp_floats,
     always_iterable,
     always_iterable_hashable,
@@ -92,11 +88,13 @@ from utilities.iterables import (
     check_superset,
     check_unique_modulo_case,
     chunked,
+    cmp_nullable,
     ensure_hashables,
     ensure_iterable,
     ensure_iterable_not_str,
     expanding_window,
     filter_include_and_exclude,
+    group_consecutive_integers,
     groupby_lists,
     hashable_to_iterable,
     is_iterable,
@@ -112,18 +110,19 @@ from utilities.iterables import (
     one_unique,
     pairwise_tail,
     product_dicts,
+    range_partitions,
     reduce_mappings,
     resolve_include_and_exclude,
     sort_iterable,
     sum_mappings,
     take,
     transpose,
+    ungroup_consecutive_integers,
     unique_everseen,
 )
 from utilities.sentinel import Sentinel, sentinel
 
 if TYPE_CHECKING:
-    import datetime as dt
     from collections.abc import Iterable, Iterator, Mapping, Sequence
 
     from utilities.types import MaybeIterable, StrMapping
@@ -611,165 +610,25 @@ class TestChunked:
         assert result == expected
 
 
-@dataclass(unsafe_hash=True, slots=True)
-class _Item:
-    n: int
-
-
-class TestCollection:
-    def test_and_singleton(self) -> None:
-        collection = Collection(map(_Item, range(3)))
-        result = collection & _Item(1)
-        assert isinstance(result, Collection)
-        expected = Collection(_Item(1))
-        assert result == expected
-
-    def test_and_collection(self) -> None:
-        collection = Collection(map(_Item, range(3)))
-        result = collection & Collection(_Item(1))
-        assert isinstance(result, Collection)
-        expected = Collection(_Item(1))
-        assert result == expected
-
-    def test_and_iterable(self) -> None:
-        collection = Collection(map(_Item, range(3)))
-        result = collection & [_Item(1)]
-        assert isinstance(result, Collection)
-        expected = Collection(_Item(1))
-        assert result == expected
-
-    def test_filter(self) -> None:
-        collection = Collection(map(_Item, range(4)))
-        result = collection.filter(lambda item: item.n % 2 == 0)
-        assert isinstance(result, Collection)
-        expected = Collection(_Item(0), _Item(2))
-        assert result == expected
-
-    def test_hash(self) -> None:
-        collection = Collection(map(_Item, range(3)))
-        _ = hash(collection)
-
-    def test_init(self) -> None:
-        class SubCollection(Collection[_Item]):
-            @override
-            def __init__(self, *item_or_items: MaybeIterable[_Item]) -> None:
-                super().__init__(*item_or_items)
-                if any(item.n >= 1 for item in self):
-                    msg = "n >= 1 is not permitted"
-                    raise ValueError(msg)
-
-        with raises(ValueError, match="n >= 1 is not permitted"):
-            _ = SubCollection(map(_Item, range(3)))
-
-    def test_map_return_same_type(self) -> None:
-        collection = Collection(map(_Item, range(3)))
-        result = collection.map(lambda item: replace(item, n=item.n + 1))
-        assert isinstance(result, Collection)
-        expected = Collection(map(_Item, range(1, 4)))
-        assert result == expected
-
-    def test_map_return_different_type(self) -> None:
-        collection = Collection(map(_Item, range(3)))
-        result = collection.map(lambda item: item.n)
-        assert isinstance(result, Collection)
-        expected = Collection(range(3))
-        assert result == expected
-
-    def test_new_one_singleton(self) -> None:
-        collection = Collection(_Item(1))
-        assert isinstance(collection, Collection)
-        assert len(collection) == 1
-        assert one(collection) == _Item(1)
-
-    def test_new_one_iterable(self) -> None:
-        collection = Collection(map(_Item, range(3)))
-        assert isinstance(collection, Collection)
-        assert len(collection) == 3
-
-    def test_new_many_singletons(self) -> None:
-        collection = Collection(_Item(1), _Item(2), _Item(3))
-        assert isinstance(collection, Collection)
-        assert len(collection) == 3
-
-    def test_new_many_iterables(self) -> None:
-        collection = Collection(map(_Item, range(3)), map(_Item, range(3)))
-        assert isinstance(collection, Collection)
-        assert len(collection) == 3
-
-    def test_new_check_items(self) -> None:
-        class SubCollection(Collection[_Item]):
-            @classmethod
-            @override
-            def check_items(cls, items: Iterable[_Item]) -> None:
-                if any(item.n >= 1 for item in items):
-                    msg = "n >= 1 is not permitted"
-                    raise ValueError(msg)
-
-        with raises(ValueError, match="n >= 1 is not permitted"):
-            _ = SubCollection(map(_Item, range(3)))
-
-    def test_or_singleton(self) -> None:
-        collection = Collection(map(_Item, range(3)))
-        result = collection | _Item(3)
-        assert isinstance(result, Collection)
-        expected = Collection(map(_Item, range(4)))
-        assert result == expected
-
-    def test_or_collection(self) -> None:
-        collection = Collection(map(_Item, range(3)))
-        result = collection | Collection(map(_Item, range(1, 4)))
-        assert isinstance(result, Collection)
-        expected = Collection(map(_Item, range(4)))
-        assert result == expected
-
-    def test_or_iterable(self) -> None:
-        collection = Collection(map(_Item, range(3)))
-        result = collection | map(_Item, range(1, 4))
-        assert isinstance(result, Collection)
-        expected = Collection(map(_Item, range(4)))
-        assert result == expected
-
-    def test_partition(self) -> None:
-        collection = Collection(map(_Item, range(4)))
-        result_false, result_true = collection.partition(lambda item: item.n % 2 == 0)
-        assert isinstance(result_false, Collection)
-        expected_false = Collection(_Item(1), _Item(3))
-        assert result_false == expected_false
-        assert isinstance(result_true, Collection)
-        expected_true = Collection(_Item(0), _Item(2))
-        assert result_true == expected_true
-
-    def test_repr(self) -> None:
-        collection = Collection(map(_Item, range(3)))
-        result = repr(collection)
-        expected = "Collection({_Item(n=0), _Item(n=1), _Item(n=2)})"
-        assert result == expected
-
-    def test_str(self) -> None:
-        collection = Collection(map(_Item, range(3)))
-        result = str(collection)
-        expected = "Collection({_Item(n=0), _Item(n=1), _Item(n=2)})"
-        assert result == expected
-
-    def test_sub_single_item(self) -> None:
-        collection = Collection(map(_Item, range(3)))
-        result = collection - _Item(1)
-        assert isinstance(result, Collection)
-        expected = Collection(_Item(0), _Item(2))
-        assert result == expected
-
-    def test_sub_collection(self) -> None:
-        collection = Collection(map(_Item, range(3)))
-        result = collection - Collection(_Item(1))
-        assert isinstance(result, Collection)
-        expected = Collection(_Item(0), _Item(2))
-        assert result == expected
-
-    def test_sub_iterable(self) -> None:
-        collection = Collection(map(_Item, range(3)))
-        result = collection - [_Item(1)]
-        assert isinstance(result, Collection)
-        expected = Collection(_Item(0), _Item(2))
+class TestCmpNullable:
+    @given(
+        data=data(),
+        case=sampled_from([
+            ([None, None], [None, None]),
+            ([1, None], [None, 1]),
+            ([1, None, None], [None, None, 1]),
+            ([2, 1, None], [None, 1, 2]),
+            ([2, 1, None, None], [None, None, 1, 2]),
+        ]),
+    )
+    def test_main(
+        self,
+        *,
+        data: DataObject,
+        case: tuple[Sequence[int | None], Sequence[int | None]],
+    ) -> None:
+        values, expected = case
+        result = sorted(data.draw(permutations(values)), key=cmp_to_key(cmp_nullable))
         assert result == expected
 
 
@@ -890,6 +749,18 @@ class TestFilterIncludeAndExclude:
         assert result == expected
 
 
+class TestGroupAndUngroupConsecutiveIntegers:
+    @given(xs=lists(integers(), unique=True).map(sorted))
+    def test_main(self, *, xs: list[int]) -> None:
+        result = list(ungroup_consecutive_integers(group_consecutive_integers(xs)))
+        assert result == xs
+
+    def test_example(self) -> None:
+        result = list(group_consecutive_integers([1, 2, 3, 6, 7, 10, 11, 12]))
+        expected = [(1, 3), (6, 7), (10, 12)]
+        assert result == expected
+
+
 class TestGroupbyLists:
     iterable: ClassVar[str] = "AAAABBBCCDAABB"
 
@@ -1044,7 +915,7 @@ class TestOne:
 
     @given(args=sampled_from([([],), ([], []), ([], [], [])]))
     def test_error_empty(self, *, args: tuple[Iterable[Any], ...]) -> None:
-        with raises(OneEmptyError, match=r"Iterable\(s\) must not be empty"):
+        with raises(OneEmptyError, match=r"Iterable\(s\) .* must not be empty"):
             _ = one(*args)
 
     @given(iterable=sets(integers(), min_size=2))
@@ -1120,52 +991,52 @@ class TestOneStr:
 
     def test_error_exact_match_case_insensitive_empty_error(self) -> None:
         with raises(
-            _OneStrEmptyError, match=r"Iterable .* does not contain 'd' \(modulo case\)"
+            OneStrEmptyError, match=r"Iterable .* does not contain 'd' \(modulo case\)"
         ):
             _ = one_str(["a", "b", "c"], "d")
 
     def test_error_exact_match_case_insensitive_non_unique_error(self) -> None:
         with raises(
-            _OneStrNonUniqueError,
+            OneStrNonUniqueError,
             match=r"Iterable .* must contain 'a' exactly once \(modulo case\); got 'a', 'A' and perhaps more",
         ):
             _ = one_str(["a", "A"], "a")
 
     def test_error_head_case_insensitive_empty_error(self) -> None:
         with raises(
-            _OneStrEmptyError,
+            OneStrEmptyError,
             match=r"Iterable .* does not contain any string starting with 'ac' \(modulo case\)",
         ):
             _ = one_str(["abc", "ade", "afg"], "ac", head=True)
 
     def test_error_head_case_insensitive_non_unique_error(self) -> None:
         with raises(
-            _OneStrNonUniqueError,
+            OneStrNonUniqueError,
             match=r"Iterable .* must contain exactly one string starting with 'ab' \(modulo case\); got 'abc', 'ABC' and perhaps more",
         ):
             _ = one_str(["abc", "ABC"], "ab", head=True)
 
     def test_error_exact_match_case_sensitive_empty_error(self) -> None:
-        with raises(_OneStrEmptyError, match=r"Iterable .* does not contain 'A'"):
+        with raises(OneStrEmptyError, match=r"Iterable .* does not contain 'A'"):
             _ = one_str(["a", "b", "c"], "A", case_sensitive=True)
 
     def test_error_exact_match_case_sensitive_non_unique(self) -> None:
         with raises(
-            _OneStrNonUniqueError,
+            OneStrNonUniqueError,
             match=r"Iterable .* must contain 'a' exactly once; got 'a', 'a' and perhaps more",
         ):
             _ = one_str(["a", "a"], "a", case_sensitive=True)
 
     def test_error_head_case_sensitive_empty_error(self) -> None:
         with raises(
-            _OneStrEmptyError,
+            OneStrEmptyError,
             match=r"Iterable .* does not contain any string starting with 'AB'",
         ):
             _ = one_str(["abc", "ade", "afg"], "AB", head=True, case_sensitive=True)
 
     def test_error_head_case_sensitive_non_unique(self) -> None:
         with raises(
-            _OneStrNonUniqueError,
+            OneStrNonUniqueError,
             match=r"Iterable .* must contain exactly one string starting with 'ab'; got 'abc', 'abd' and perhaps more",
         ):
             _ = one_str(["abc", "abd"], "ab", head=True, case_sensitive=True)
@@ -1216,6 +1087,66 @@ class TestProductDicts:
         assert result == expected
 
 
+class TestRangePartitions:
+    @given(
+        case=sampled_from([
+            (1, 0, 1, [0]),
+            (2, 0, 1, [0, 1]),
+            (2, 0, 2, [0]),
+            (2, 1, 2, [1]),
+            (3, 0, 1, [0, 1, 2]),
+            (3, 0, 2, [0, 1]),
+            (3, 1, 2, [2]),
+            (3, 0, 3, [0]),
+            (3, 1, 3, [1]),
+            (3, 2, 3, [2]),
+            (6, 0, 1, [0, 1, 2, 3, 4, 5]),
+            (6, 0, 2, [0, 1, 2]),
+            (6, 1, 2, [3, 4, 5]),
+            (6, 0, 3, [0, 1]),
+            (6, 1, 3, [2, 3]),
+            (6, 2, 3, [4, 5]),
+            (7, 0, 2, [0, 1, 2, 3]),
+            (7, 1, 2, [4, 5, 6]),
+            (7, 0, 3, [0, 1, 2]),
+            (7, 1, 3, [3, 4]),
+            (7, 2, 3, [5, 6]),
+        ])
+    )
+    def test_main(self, *, case: tuple[int, int, int, Sequence[int]]) -> None:
+        stop, num, total, expected = case
+        result = list(range_partitions(stop, num, total))
+        assert result == expected
+
+    def test_error_stop(self) -> None:
+        with raises(_RangePartitionsStopError, match="'stop' must be positive; got 0"):
+            _ = range_partitions(0, 0, 0)
+
+    def test_error_total_too_low(self) -> None:
+        with raises(
+            _RangePartitionsTotalError, match=r"'total' must be in \[1, 1\]; got 0"
+        ):
+            _ = range_partitions(1, 0, 0)
+
+    def test_error_total_too_high(self) -> None:
+        with raises(
+            _RangePartitionsTotalError, match=r"'total' must be in \[1, 1\]; got 2"
+        ):
+            _ = range_partitions(1, 0, 2)
+
+    def test_error_num_too_low(self) -> None:
+        with raises(
+            _RangePartitionsNumError, match=r"'num' must be in \[0, 1\]; got -1"
+        ):
+            _ = range_partitions(2, -1, 2)
+
+    def test_error_num_too_high(self) -> None:
+        with raises(
+            _RangePartitionsNumError, match=r"'num' must be in \[0, 1\]; got 2"
+        ):
+            _ = range_partitions(2, 2, 2)
+
+
 class TestReduceMappings:
     @given(
         mappings=lists(dictionaries(text_ascii(), integers())),
@@ -1274,10 +1205,9 @@ class TestResolveIncludeAndExclude:
 
 
 class TestSortIterable:
-    @given(
-        x=make_objects(floats_allow_nan=False), y=make_objects(floats_allow_nan=False)
-    )
-    def test_main(self, *, x: Any, y: Any) -> None:
+    @given(objs=pairs(objects(floats_allow_nan=False, sortable=True)))
+    def test_main(self, *, objs: tuple[Any, Any]) -> None:
+        x, y = objs
         result1 = sort_iterable([x, y])
         result2 = sort_iterable([y, x])
         assert result1 == result2
@@ -1324,14 +1254,6 @@ class TestSortIterable:
             _ = sort_iterable([sentinel, sentinel])
 
 
-class TestSortIterablesCmpDateTimes:
-    @given(x=datetimes() | zoned_datetimes(), y=datetimes() | zoned_datetimes())
-    def test_main(self, *, x: dt.datetime, y: dt.datetime) -> None:
-        result1 = _sort_iterable_cmp_datetimes(x, y)
-        result2 = _sort_iterable_cmp_datetimes(y, x)
-        assert result1 == -result2
-
-
 class TestSortIterablesCmpFloats:
     @given(x=floats(), y=floats())
     def test_main(self, *, x: float, y: float) -> None:
@@ -1345,6 +1267,20 @@ class TestSortIterablesCmpFloats:
         if isnan(x) is not isnan(y):
             assert result1 != result2
 
+    @given(x=floats(allow_nan=False))
+    def test_num_vs_nan(self, *, x: float) -> None:
+        result = _sort_iterable_cmp_floats(x, nan)
+        assert result == -1
+
+    @given(x=floats(allow_nan=False))
+    def test_nan_vs_num(self, *, x: float) -> None:
+        result = _sort_iterable_cmp_floats(nan, x)
+        assert result == 1
+
+    def test_nan_vs_nan(self) -> None:
+        result = _sort_iterable_cmp_floats(nan, nan)
+        assert result == 0
+
 
 class TestSumMappings:
     @given(mappings=lists(dictionaries(text_ascii(), integers())))
