@@ -5,14 +5,15 @@ from functools import partial, wraps
 from inspect import iscoroutinefunction
 from os import environ
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, ParamSpec, assert_never, cast, override
+from typing import TYPE_CHECKING, Any, assert_never, cast, override
 
-from pytest import fixture
+from whenever import ZonedDateTime
 
-from utilities.datetime import datetime_duration_to_float, get_now
+from utilities.atomicwrites import writer
 from utilities.functools import cache
 from utilities.hashlib import md5_hash
-from utilities.pathlib import ensure_suffix
+from utilities.os import get_env_var
+from utilities.pathlib import ensure_suffix, get_root, get_tail, module_path
 from utilities.platform import (
     IS_LINUX,
     IS_MAC,
@@ -21,25 +22,20 @@ from utilities.platform import (
     IS_NOT_WINDOWS,
     IS_WINDOWS,
 )
-from utilities.random import get_state
+from utilities.types import MaybeCoro
+from utilities.whenever import SECOND, get_now_local
 
 if TYPE_CHECKING:
     from collections.abc import Callable, Iterable, Sequence
-    from random import Random
 
-    from utilities.types import (
-        Coroutine1,
-        Duration,
-        PathLike,
-        TCallableMaybeCoroutine1None,
-    )
+    from whenever import TimeDelta
+
+    from utilities.types import Coro, PathLike
 
 try:  # WARNING: this package cannot use unguarded `pytest` imports
-    from _pytest.config import Config  # pyright: ignore[reportPrivateImportUsage]
-    from _pytest.config.argparsing import (
-        Parser,  # pyright: ignore[reportPrivateImportUsage]
-    )
-    from _pytest.python import Function  # pyright: ignore[reportPrivateImportUsage]
+    from _pytest.config import Config
+    from _pytest.config.argparsing import Parser
+    from _pytest.python import Function
     from pytest import mark, skip
 except ModuleNotFoundError:  # pragma: no cover
     from typing import Any as Config
@@ -58,12 +54,6 @@ else:
     skipif_not_linux = mark.skipif(IS_NOT_LINUX, reason="Skipped for non-Linux")
 
 
-_P = ParamSpec("_P")
-
-
-##
-
-
 def add_pytest_addoption(parser: Parser, options: Sequence[str], /) -> None:
     """Add the `--slow`, etc options to pytest.
 
@@ -131,18 +121,18 @@ def is_pytest() -> bool:
 ##
 
 
-def node_id_to_path(
-    node_id: str, /, *, head: PathLike | None = None, suffix: str | None = None
+def node_id_path(
+    node_id: str, /, *, root: PathLike | None = None, suffix: str | None = None
 ) -> Path:
-    """Map a node ID to a path."""
+    """Get the path of a node ID."""
     path_file, *parts = node_id.split("::")
     path_file = Path(path_file)
     if path_file.suffix != ".py":
         raise NodeIdToPathError(node_id=node_id)
     path = path_file.with_suffix("")
-    if head is not None:
-        path = path.relative_to(head)
-    path = Path(".".join(path.parts), "__".join(parts))
+    if root is not None:
+        path = get_tail(path, root)
+    path = Path(module_path(path), "__".join(parts))
     if suffix is not None:
         path = ensure_suffix(path, suffix)
     return path
@@ -160,100 +150,106 @@ class NodeIdToPathError(Exception):
 ##
 
 
-@fixture
-def random_state(*, seed: int) -> Random:
-    """Fixture for a random state."""
-    return get_state(seed=seed)
-
-
-##
-
-
-def throttle(
-    *, root: PathLike | None = None, duration: Duration = 1.0, on_try: bool = False
-) -> Callable[[TCallableMaybeCoroutine1None], TCallableMaybeCoroutine1None]:
+def throttle[F: Callable[..., MaybeCoro[None]]](
+    *, root: PathLike | None = None, delta: TimeDelta = SECOND, on_try: bool = False
+) -> Callable[[F], F]:
     """Throttle a test. On success by default, on try otherwise."""
-    root_use = Path(".pytest_cache", "throttle") if root is None else Path(root)
-    return cast(
-        "Any", partial(_throttle_inner, root=root_use, duration=duration, on_try=on_try)
-    )
+    return cast("Any", partial(_throttle_inner, root=root, delta=delta, on_try=on_try))
 
 
-def _throttle_inner(
-    func: TCallableMaybeCoroutine1None,
+def _throttle_inner[F: Callable[..., MaybeCoro[None]]](
+    func: F,
     /,
     *,
-    root: Path,
-    duration: Duration = 1.0,
+    root: PathLike | None = None,
+    delta: TimeDelta = SECOND,
     on_try: bool = False,
-) -> TCallableMaybeCoroutine1None:
+) -> F:
     """Throttle a test function/method."""
-    match bool(iscoroutinefunction(func)):
-        case False:
-            func_typed = cast("Callable[..., None]", func)
+    if get_env_var("THROTTLE", nullable=True) is not None:
+        return func
+    match bool(iscoroutinefunction(func)), on_try:
+        case False, False:
 
             @wraps(func)
-            def throttle_sync(*args: _P.args, **kwargs: _P.kwargs) -> None:
-                """Call the throttled sync test function/method."""
-                path, now = _throttle_path_and_now(root, duration=duration)
-                if on_try:
-                    _throttle_write(path, now)
-                    return func_typed(*args, **kwargs)
-                func_typed(*args, **kwargs)
-                _throttle_write(path, now)
-                return None
+            def throttle_sync_on_pass(*args: Any, **kwargs: Any) -> None:
+                _skipif_recent(root=root, delta=delta)
+                cast("Callable[..., None]", func)(*args, **kwargs)
+                _write(root=root)
 
-            return cast("TCallableMaybeCoroutine1None", throttle_sync)
-        case True:
-            func_typed = cast("Callable[..., Coroutine1[None]]", func)
+            return cast("Any", throttle_sync_on_pass)
+
+        case False, True:
 
             @wraps(func)
-            async def throttle_async(*args: _P.args, **kwargs: _P.kwargs) -> None:
-                """Call the throttled async test function/method."""
-                path, now = _throttle_path_and_now(root, duration=duration)
-                if on_try:
-                    _throttle_write(path, now)
-                    return await func_typed(*args, **kwargs)
-                await func_typed(*args, **kwargs)
-                _throttle_write(path, now)
-                return None
+            def throttle_sync_on_try(*args: Any, **kwargs: Any) -> None:
+                _skipif_recent(root=root, delta=delta)
+                _write(root=root)
+                cast("Callable[..., None]", func)(*args, **kwargs)
+
+            return cast("Any", throttle_sync_on_try)
+
+        case True, False:
+
+            @wraps(func)
+            async def throttle_async_on_pass(*args: Any, **kwargs: Any) -> None:
+                _skipif_recent(root=root, delta=delta)
+                await cast("Callable[..., Coro[None]]", func)(*args, **kwargs)
+                _write(root=root)
+
+            return cast("Any", throttle_async_on_pass)
+
+        case True, True:
+
+            @wraps(func)
+            async def throttle_async_on_try(*args: Any, **kwargs: Any) -> None:
+                _skipif_recent(root=root, delta=delta)
+                _write(root=root)
+                await cast("Callable[..., Coro[None]]", func)(*args, **kwargs)
+
+            return cast("Any", throttle_async_on_try)
 
-            return cast("TCallableMaybeCoroutine1None", throttle_async)
         case _ as never:
             assert_never(never)
 
 
-def _throttle_path_and_now(
-    root: Path, /, *, duration: Duration = 1.0
-) -> tuple[Path, float]:
-    test = environ["PYTEST_CURRENT_TEST"]
-    path = Path(root, _throttle_md5_hash(test))
-    if path.exists():
-        with path.open(mode="r") as fh:
-            contents = fh.read()
-        prev = float(contents)
+def _skipif_recent(*, root: PathLike | None = None, delta: TimeDelta = SECOND) -> None:
+    if skip is None:
+        return  # pragma: no cover
+    path = _get_path(root=root)
+    try:
+        contents = path.read_text()
+    except FileNotFoundError:
+        return
+    try:
+        last = ZonedDateTime.parse_common_iso(contents)
+    except ValueError:
+        return
+    if (age := (get_now_local() - last)) < delta:
+        _ = skip(reason=f"{_get_name()} throttled (age {age})")
+
+
+def _get_path(*, root: PathLike | None = None) -> Path:
+    if root is None:
+        root_use = get_root().joinpath(".pytest_cache", "throttle")  # pragma: no cover
     else:
-        prev = None
-    now = get_now().timestamp()
-    if (
-        (skip is not None)
-        and (prev is not None)
-        and ((now - prev) < datetime_duration_to_float(duration))
-    ):
-        _ = skip(reason=f"{test} throttled")
-    return path, now
+        root_use = root
+    return Path(root_use, _md5_hash_cached(_get_name()))
 
 
 @cache
-def _throttle_md5_hash(text: str, /) -> str:
+def _md5_hash_cached(text: str, /) -> str:
     return md5_hash(text)
 
 
-def _throttle_write(path: Path, now: float, /) -> None:
-    from utilities.atomicwrites import writer
+def _get_name() -> str:
+    return environ["PYTEST_CURRENT_TEST"]
 
-    with writer(path, overwrite=True) as temp, temp.open(mode="w") as fh:
-        _ = fh.write(str(now))
+
+def _write(*, root: PathLike | None = None) -> None:
+    path = _get_path(root=root)
+    with writer(path, overwrite=True) as temp:
+        _ = temp.write_text(get_now_local().format_common_iso())
 
 
 __all__ = [
@@ -262,8 +258,7 @@ __all__ = [
     "add_pytest_collection_modifyitems",
     "add_pytest_configure",
     "is_pytest",
-    "node_id_to_path",
-    "random_state",
+    "node_id_path",
     "skipif_linux",
     "skipif_mac",
     "skipif_not_linux",
